/*
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @author $Author: leine $
 * @version $Rev: 72369 $
 * @base Legato.Control
 * @requires Legato/Control.js
 */

/**
 * Class: Legato.Control.WMSGetFeatureInfo
 *
 * This class represents a compositum of the
 * OpenLayers.Control.WMSGetFeatureInfo class and the
 * OpenLayers.Popup.FramedCloud. If the GetFeatureInfo reuest is made the
 * response will been rendered as a popup.
 *
 * Inherits from: - <Legato.Control>
 */
Legato.Control.WMSGetFeatureInfo = OpenLayers.Class(Legato.Control, {
  /**
   * Property: wmsGetFeatureInfo {<OpenLayers.Control.WMSGetFeatureInfo>} the
   * WMSGetFeatureInfo instance which will been used for requesting
   */
  wmsGetFeatureInfo :null,

  /**
   * Property: popup {<OpenLayers.Popup>} The popup which will render the
   * reponse
   */
  popup :null,

  /**
   * Property: popupType {<String>} The popuptype which will been used to
   * render the response. Possible values are Anchored, AnchroedBubble and
   * FramedCloud
   */
  popupType :'',

  /**
   * Property: autoActive {<Boolean>} If set to true the control will allways
   * be enabled and active, meaning that if the user hovers over the map the
   * control will always perform a WMSGetFeatureInfo request.
   */
  autoActive :false,

  /**
   * Property: popupBackgroundColor {<String>} Defines the backgroundColor of the
   * popup. Not used if popuptype is set to AnchroedBubble, cause you can not
   * set the backgroundColor for this type.
   */
  popupBackgroundColor :'',

  /*
   * Property: popupMinSize
   *
   * See Also: <OpenLayers.Popup.minSize>
   */
  popupMinSize : null,

  /*
   * Property: popupMaxSize
   *
   * See Also: <OpenLayers.Popup.maxSize>
   */
  popupMaxSize : null,

  /*
   * Property: popupAutoSize
   *
   * See Also: <OpenLayers.Popup.maxSize>
   */
  popupAutoSize: true,

  /*
   * Property: popupPanMapIfOutOfView
   *
   * See Also: <OpenLayers.Popup.panMapIfOutOfView>
   */
  popupPanMapIfOutOfView: true,

  /*
   * Property: popupKeepInMap
   *
   * See Also: <OpenLayers.Popup.keepInMap>
   */
  popupKeepInMap:  true,

  /*
   * Property: popupCloseOnMove
   *
   * See Also: <OpenLayers.Popup.closeOnMove>
   */
  popupCloseOnMove: false,

  /*
   * Property: popupOpacity
   *
   * See Also: <OpenLayers.Popup.opacity>
   */
  popupOpacity :null,


  /*
   * Property: renderIfNoPopupExists
   *
   * Only render if no other popup exists
   */
  renderIfNoPopupExists: false,

  /**
   * Constructor: Legato.Control.WMSGetFeatureInfo Create a WMSGetFeatureInfo.
   *
   * Parameters:
   */
  initialize : function(options) {
    if (!options.delay) {
      options.delay = 500;
    }

    if (!options.popupBackgroundColor) {
      options.popupBackgroundColor = '#fafad2';
    }

    options.handlerOptions = {
      click :options.delay,
      hover :options.delay
    };

    Legato.Control.prototype.initialize.apply(this, [ options ]);
    this.wmsGetFeatureInfo = new OpenLayers.Control.WMSGetFeatureInfo(options);
  },

  /**
   * Function: draw
   *
   * See Also: <OpenLayer.Control>
   */
  draw : function() {
    OpenLayers.Control.prototype.draw.apply(this);
  },

  enableAutoActive : function() {
    if (!this.wmsGetFeatureInfo.active) {
      this.activate();
    }
  },

  /**
   * APIMethod: destroy
   *
   * See Also: <OpenLayer.Control>
   */
  destroy : function() {
    if (this.autoActive) {
      this.map.events.un( {
        'mouseover' :this.enableAutoActive,
        scope :this
      });
    }

    if (this.hover) {
      this.map.events.un( {
        'mousemove' :this.hidePopup,
        scope :this
      });
    }

    this.map.events.un( {
      'legato.selectlayer'    : this.updateQueryLayers,
      'legato.deselectlayer'  : this.updateQueryLayers,
      'addlayer'              : this.updateQueryLayers,
      'removelayer'           : this.updateQueryLayers,
      scope :this
    });

    this.wmsGetFeatureInfo.events.un( {
      'getfeatureinfo' :this.renderGetFeatureInfo,
      scope :this
    });

    this.wmsGetFeatureInfo.destroy();

    if (this.popup !== null) {
      this.map.removePopup(this.popup);
      this.popup.destroy();
    }

    OpenLayers.Control.prototype.destroy.apply(this, arguments);
  },

  /**
   * APIMethod: activate
   *
   * See Also: <OpenLayer.Control>
   */
  activate : function() {
    this.updateQueryLayers();
    this.wmsGetFeatureInfo.activate();
    OpenLayers.Control.prototype.activate.apply(this, arguments);
  },

  /**
   * APIMethod: deactivate
   *
   * See Also: <OpenLayer.Control>
   */
  deactivate : function() {
    this.wmsGetFeatureInfo.deactivate();
    this.hidePopup();
    OpenLayers.Control.prototype.deactivate.apply(this, arguments);
  },

  /**
   * Hides the current popup if exists
   */
  hidePopup : function() {
    if (this.popup !== null) {
      this.popup.hide();
    }
  },

  /**
   * Returns all current selectedlayers
   */
  getSelectedLayers: function() {
    return Legato.Lang.CollectionUtils.filter(this.map.layers,
        function(layer) {
          return Legato.OpenLayers.Map.Util.isListedLayer(layer)
              && Legato.Lang.ObjectUtils.isTrue(layer.selected)
              && Legato.Lang.ObjectUtils.isTrue(layer.queryable);
    });
  },

  /**
   * Returns all queryable layers
   */
  getQueryAbleLayers: function(){
    return Legato.Lang.CollectionUtils.filter(this.map.layers,
        function(layer) {
          return Legato.OpenLayers.Map.Util.isListedLayer(layer)
              && Legato.Lang.ObjectUtils.isTrue(layer.queryable);
    });
  },

  /**
   * Returns all layers
   */
  getListedLayers: function(){
    return Legato.Lang.CollectionUtils.filter(this.map.layers,
        function(layer) {
          return Legato.OpenLayers.Map.Util.isListedLayer(layer);
    });
  },

  /**
   * Updates all current layers by injecting them into the wrapped
   * WMSGetFeatureInfo control of openlayers.
   */
  updateQueryLayers : function() {

    var selectedLayers = this.getSelectedLayers();

    this.wmsGetFeatureInfo.vendorParams = {
        selected_layers: Legato.OpenLayers.Map.Util.getLayerIds(selectedLayers)
    };

    this.wmsGetFeatureInfo.layers = this.getQueryAbleLayers();

  },
  /*
   * APIMethod: setMap
   *
   * See Also: <OpenLayer.Control>
   */
  setMap : function(map) {
    this.wmsGetFeatureInfo.setMap(map);
    OpenLayers.Control.prototype.setMap.apply(this, arguments);

    if (this.autoActive) {
      this.map.events.on( {
        'mouseover' :this.enableAutoActive,
        scope :this
      });
    }

    if (this.hover) {
      this.map.events.on( {
        'mousemove' :this.hidePopup,
        scope :this
      });
    }

    this.map.events.on( {
      'legato.selectlayer'    : this.updateQueryLayers,
      'legato.deselectlayer'  : this.updateQueryLayers,
      'addlayer'              : this.updateQueryLayers,
      'removelayer'           : this.updateQueryLayers,
      scope :this
    });

    this.wmsGetFeatureInfo.events.on( {
      'getfeatureinfo' :this.renderGetFeatureInfo,
      scope :this
    });
  },

  /*
   * renders the returned response from the GetFeatureInfo request inside a
   * popup.
   */
  renderGetFeatureInfo : function(featureInfo) {
    if (this.popup !== null) {
      this.map.removePopup(this.popup);
    }

    /*
     * Just render a popup if there is some response
     */
    if (featureInfo.text.length === 0) {
      return;
    }

    /*
     * Abort showing the popup if another popup is allready shown
     * and the config for this control is set to true
     */
    if(this.renderIfNoPopupExists && this.map.popups.length > 0){
      return;
    }

    switch (this.popupType.toLowerCase()) {
    case 'anchored':
      this.popup = new OpenLayers.Popup.Anchored('getFeatureInfo', this.map
          .getLonLatFromPixel(featureInfo.xy), null, featureInfo.text, null,
          !(this.autoActive || this.hover), OpenLayers.Function.bind(this.onClosePopup, this));
      break;
    case 'anchoredbubble':
      this.popup = new OpenLayers.Popup.AnchoredBubble('getFeatureInfo',
          this.map.getLonLatFromPixel(featureInfo.xy), null, featureInfo.text,
          null, !(this.autoActive || this.hover), OpenLayers.Function.bind(this.onClosePopup, this));
      break;
    //case 'framedcloud':
    default:
      this.popup = new OpenLayers.Popup.MarginAwareFramedCloud('getFeatureInfo', this.map
          .getLonLatFromPixel(featureInfo.xy), null, featureInfo.text, null,
          !(this.autoActive || this.hover), OpenLayers.Function.bind(this.onClosePopup, this));
      break;
    }

    //apply popup options
    this.popup.autoSize = this.popupAutoSize;
    this.popup.setBackgroundColor(this.popupBackgroundColor);
    if (this.popupMinSize) {
      this.popup.minSize = this.popupMinSize;
    }
    if (this.popupMaxSize) {
      this.popup.maxSize = this.popupMaxSize;
    }
    if (this.popupOpacity) {
      this.popup.setOpacity(this.popupOpacity);
    }
    this.popup.panMapIfOutOfView = this.popupPanMapIfOutOfView;
    this.popup.keepInMap = this.popupKeepInMap;
    this.popup.closeOnMove = this.popupCloseOnMove;

    this.map.addPopup(this.popup);
  },

  /*
   * Callback function. Triggered whenever the user clicks its close button
   */
  onClosePopup: function(event){
    this.map.removePopup(this.popup);
  },

  CLASS_NAME :'Legato.Control.WMSGetFeatureInfo'
});

/**
 * Structure: lc:WMSGetFeatureInfo XML based config for a
 * <Legato.Control.WMSGetFeatureInfo>.
 *
 * See Also: - <Legato.Control> - <Legato.Beans.BeanFactory> - <QName>
 *
 * A valid config example for a WMSGetFeatureInfo would be:
 * (start code)
 * <lc:WMSGetFeatureInfo>
 *   <infoFormat>text/xml</infoFormat>
 *   <hover>true</hover>
 *   <maxFeatures>1</maxFeatures>
 *   <queryVisible>true</queryVisible>
 *   <handlerOptions>
 *   <hover>
 *    <delay>200</delay>
 *  </hover>
 *  </handlerOptions>
 * </lc:WMSGetFeatureInfo>
 * (end)
 */
Legato.Control.WMSGetFeatureInfo.Bean = Legato.Control.Bean.Extend(

'Legato.Control.WMSGetFeatureInfo',

/*
 * Property: QName The qualified name for this control. Needed by XML config to
 * instantiate it.
 */
Legato.Control.QName('WMSGetFeatureInfo'), {
  _constructor :Legato.Control.WMSGetFeatureInfo,
  options : {
    /*
     * Property: hover
     *
     * See Also: <OpenLayers.Control.WMSGetFeatureInfo.hover>
     */
    hover :Legato.Lang.Boolean,

    /*
     * Property: maxFeatures
     *
     * See Also: <OpenLayers.Control.WMSGetFeatureInfo.maxFeatures>
     */
    maxFeatures :Legato.Lang.Integer,

    /*
     * Property: queryVisible
     *
     * See Also: <OpenLayers.Control.WMSGetFeatureInfo.queryVisible>
     */
    queryVisible :Legato.Lang.Boolean,

    /*
     * Property: url
     *
     * See Also: <OpenLayers.Control.WMSGetFeatureInfo.url>
     */
    url :Legato.Lang.String,

    /*
     * Property: infoFormat
     *
     * See Also: <OpenLayers.Control.WMSGetFeatureInfo.infoFormat>
     */
    infoFormat :Legato.Lang.String,

    /*
     * Property: autoActive If set to true the control will allways be enabled
     * and active, meaning that if the user hovers over the map the control will
     * always perform a WMSGetFeatureInfo request.
     */
    autoActive :Legato.Lang.Boolean,

    /*
     * Property: renderIfNoPopupExists If set to true the control will just render itself
     * when there is currently no other popup shown.
     */
    renderIfNoPopupExists :Legato.Lang.Boolean,

    /*
     * Property: delay Defines the delay in ms where a GetFeatureInfoRequest is
     * made. Default is 500 ms.
     */
    delay :Legato.Lang.Integer,

    /*
     * Property: popupType Defines the popuptype which will been used to render
     * the response. Possible values are Anchored, AnchroedBubble and
     * FramedCloud
     */
    popupType :Legato.Lang.String,

    /*
     * Property: popupBackgroundColor Defines the backgroundColor of the popup. Not
     * used if popuptype is set to AnchroedBubble, cause you can not set the
     * backgroundColor for this type.
     */
    popupBackgroundColor :Legato.Lang.String,

    /*
     * Property: popupMinSize
     *
     * See Also: <OpenLayers.Popup.minSize>
     */
    popupMinSize :OpenLayers.Size.Bean,

    /*
     * Property: popupMaxSize
     *
     * See Also: <OpenLayers.Popup.maxSize>
     */
    popupMaxSize :OpenLayers.Size.Bean,

    /*
     * Property: popupAutoSize
     *
     * See Also: <OpenLayers.Popup.maxSize>
     */
    popupAutoSize: Legato.Lang.Boolean,

    /*
     * Property: popupPanMapIfOutOfView
     *
     * See Also: <OpenLayers.Popup.panMapIfOutOfView>
     */
    popupPanMapIfOutOfView: Legato.Lang.Boolean,

    /*
     * Property: popupKeepInMap
     *
     * See Also: <OpenLayers.Popup.keepInMap>
     */
    popupKeepInMap:  Legato.Lang.Boolean,

    /*
     * Property: popupCloseOnMove
     *
     * See Also: <OpenLayers.Popup.closeOnMove>
     */
    popupCloseOnMove: Legato.Lang.Boolean,

    /*
     * Property: popupOpacity
     *
     * See Also: <OpenLayers.Popup.opacity>
     */
    popupOpacity :Legato.Lang.Number

  }
});